export var ChannelName;
(function (ChannelName) {
    ChannelName["Call"] = "call";
    ChannelName["Chat"] = "chat";
    ChannelName["Task"] = "task";
})(ChannelName || (ChannelName = {}));
export class Task {
    constructor(client, e, distribute) {
        this.client = client;
        this.distribute = distribute;
        this.id = e.attempt_id;
        this.state = e.status;
        this.lastStatusChange = e.timestamp;
        this.createdAt = e.timestamp;
        this._processing = null;
        this.offeringAt = 0;
        this.answeredAt = 0;
        this.bridgedAt = 0;
        this.startProcessingAt = 0;
        this.stopAt = 0;
        this.closedAt = 0;
        this.communication = distribute.communication;
        this.history = [distribute];
    }
    get queue() {
        return {
            id: this.distribute.queue_id,
            name: this.distribute.queue_name,
        };
    }
    get duration() {
        return Math.round((Date.now() - this.lastStatusChange) / 1000);
    }
    get channel() {
        return this.distribute.channel;
    }
    get allowAccept() {
        return (this.channel === ChannelName.Task &&
            (this.bridgedAt === 0 && this.closedAt === 0));
    }
    get allowDecline() {
        return this.allowAccept;
    }
    get allowClose() {
        return (this.channel === ChannelName.Task &&
            this.closedAt === 0 &&
            this.bridgedAt > 0);
    }
    get memberId() {
        return this.distribute.member_id;
    }
    get queueId() {
        return this.distribute.queue_id;
    }
    get hasReporting() {
        return this.distribute.has_reporting;
    }
    get allowReporting() {
        return this.hasReporting && this.bridgedAt > 0;
    }
    get agentChannelId() {
        return this.distribute.agent_channel_id;
    }
    setAnswered(t) {
        this.answeredAt = t;
        this.lastStatusChange = Date.now();
    }
    setBridged(t) {
        this.bridgedAt = t;
        this.lastStatusChange = Date.now();
    }
    setProcessing(p) {
        if (!this.startProcessingAt) {
            this.startProcessingAt = Date.now();
        }
        if (p.sec) {
            p.timeout = Date.now() + p.sec * 1000; // bug
        }
        this._processing = p;
    }
    setTransferred(d) {
        this.distribute = d;
        this.history.push(d);
    }
    get processingTimeoutAt() {
        if (!this._processing || !this._processing.timeout) {
            return null;
        }
        return this._processing.timeout;
    }
    get processingSec() {
        if (!this._processing || !this._processing.sec) {
            return null;
        }
        return this._processing.sec;
    }
    get renewalSec() {
        if (!this._processing || !this._processing.renewal_sec) {
            return null;
        }
        return this._processing.renewal_sec;
    }
    /*
      control
     */
    async accept() {
        return this.client.request(`cc_agent_task_accept`, {
            agent_id: this.distribute.agent_id,
            attempt_id: this.id,
            app_id: this.distribute.app_id,
        });
    }
    async close() {
        return this.client.request(`cc_agent_task_close`, {
            agent_id: this.distribute.agent_id,
            attempt_id: this.id,
            app_id: this.distribute.app_id,
        });
    }
    // todo
    async decline() {
        return this.client.request(`cc_agent_task_close`, {
            agent_id: this.distribute.agent_id,
            attempt_id: this.id,
            app_id: this.distribute.app_id,
        });
    }
    async reporting(reporting) {
        return this.client.request('cc_reporting', {
            attempt_id: this.id,
            ...reporting,
        });
    }
    async renew(sec) {
        return this.client.request('cc_renewal', {
            attempt_id: this.id,
            renewal_sec: sec ? sec : this.processingSec,
        });
    }
}
//# sourceMappingURL=task.js.map