import { __assign, __awaiter, __extends, __generator } from "tslib";
import { EventEmitter } from 'ee-ts';
import * as JsSip from 'jssip';
import { Log } from '../../log';
import { Session } from './session';
var SipPhone = /** @class */ (function (_super) {
    __extends(SipPhone, _super);
    function SipPhone(instanceId, debug) {
        var _this = _super.call(this) || this;
        _this.instanceId = instanceId;
        _this.type = 'webrtc';
        _this.sessionCache = new Map();
        if (debug) {
            JsSip.debug.enable('*');
        }
        else {
            JsSip.debug.disable();
        }
        _this.log = new Log();
        _this.on('unregistered', function () {
            _this.sessionCache.clear();
        });
        return _this;
    }
    SipPhone.prototype.callOption = function (req) {
        if (req === void 0) { req = {}; }
        return __awaiter(this, void 0, void 0, function () {
            var mediaStream;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, this.getUserCallStream(req)];
                    case 1:
                        mediaStream = _a.sent();
                        return [2 /*return*/, {
                                sessionTimersExpires: 120,
                                pcConfig: req.disableStun
                                    ? undefined
                                    : {
                                        iceServers: [{ urls: ['stun:stun.l.google.com:19302'] }],
                                    },
                                mediaStream: mediaStream,
                            }];
                }
            });
        });
    };
    SipPhone.prototype.answer = function (id, req) {
        return __awaiter(this, void 0, void 0, function () {
            var session, params;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        if (!this.sessionCache.has(id)) return [3 /*break*/, 2];
                        session = this.sessionCache.get(id);
                        return [4 /*yield*/, this.callOption(req)];
                    case 1:
                        params = _a.sent();
                        return [2 /*return*/, session.answer(params)];
                    case 2: return [2 /*return*/, false];
                }
            });
        });
    };
    SipPhone.prototype.call = function (req) {
        return __awaiter(this, void 0, void 0, function () {
            var params, display, invite;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        params = {};
                        display = null;
                        if (req.params) {
                            params.audio = req.params.audio;
                            params.video = req.params.video || false;
                            params.screen = req.params.screen || false;
                            params.disableStun = req.params.disableStun || false;
                        }
                        if (typeof req.destination !== 'string') {
                            throw new Error('bad destination type');
                        }
                        return [4 /*yield*/, this.callOption(params)];
                    case 1:
                        invite = _a.sent();
                        if (req.params && req.params.display) {
                            display = {
                                extraHeaders: ["X-Webitel-Display: " + req.params.display],
                            };
                        }
                        return [4 /*yield*/, this.ua.call(req.destination, __assign(__assign({}, invite), display))];
                    case 2:
                        _a.sent();
                        return [2 /*return*/];
                }
            });
        });
    };
    SipPhone.prototype.register = function (sipConf) {
        return __awaiter(this, void 0, void 0, function () {
            var socket, configuration, ua;
            var _this = this;
            return __generator(this, function (_a) {
                socket = new JsSip.WebSocketInterface(sipConf.server);
                configuration = {
                    realm: sipConf.realm,
                    uri: sipConf.uri,
                    authorization_user: sipConf.authorization_user,
                    ha1: sipConf.ha1,
                    user_agent: SipPhone.userAgent,
                    sockets: [socket],
                    session_timers: true,
                    // use_preloaded_route: true,
                    display_name: sipConf.display_name,
                    register_expires: sipConf.register_expires || 90,
                    connection_recovery_min_interval: 5,
                    connection_recovery_max_interval: 60,
                };
                ua = (this.ua = new JsSip.UA(configuration));
                // ua.on('connected', (e: object) => {})
                ua.on('newRTCSession', function (e) {
                    var session = e.session;
                    var id = session.id;
                    var callSession = new Session(e);
                    _this.storeSession(id, callSession);
                    session.on('connecting', function () {
                        _this.emit('localStreams', callSession, session.connection.getLocalStreams());
                    });
                    if (session.connection) {
                        session.connection.addEventListener('addstream', function (evt) {
                            _this.emit('peerStreams', callSession, new Array(evt.stream));
                        });
                    }
                    session.on('peerconnection', function (peer) {
                        peer.peerconnection.addEventListener('addstream', function (evt) { return __awaiter(_this, void 0, void 0, function () {
                            return __generator(this, function (_a) {
                                // set remote audio stream
                                this.emit('peerStreams', callSession, new Array(evt.stream));
                                return [2 /*return*/];
                            });
                        }); });
                    });
                    session.on('ended', function () {
                        // this handler will be called for incoming calls too
                        _this.removeSession(id, session.connection);
                    });
                    session.on('failed', function () {
                        // this handler will be called for incoming calls too
                        _this.removeSession(id, session.connection);
                    });
                    session.on('accepted', function () {
                        // the call has answered
                        if (!callSession.incoming) {
                            _this.emit('peerStreams', callSession, session.connection.getRemoteStreams());
                        }
                    });
                    session.on('confirmed', function () {
                        // this handler will be called for incoming calls too
                    });
                    _this.emit('newSession', callSession);
                });
                ua.on('disconnected', function (e) {
                    _this.emit('unregistered');
                });
                // ua.on('registered', (e: object) => {})
                ua.on('unregistered', function (e) {
                    _this.emit('unregistered');
                });
                ua.on('registrationFailed', function (e) {
                    _this.log.error('registrationFailed', e);
                });
                ua.on('registered', function (e) {
                    _this.emit('registered');
                });
                ua.start();
                return [2 /*return*/];
            });
        });
    };
    SipPhone.prototype.unregister = function () {
        return __awaiter(this, void 0, void 0, function () {
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        if (!this.ua) return [3 /*break*/, 2];
                        this.ua.removeAllListeners();
                        return [4 /*yield*/, this.ua.stop()];
                    case 1:
                        _a.sent();
                        delete this.ua;
                        _a.label = 2;
                    case 2: return [2 /*return*/];
                }
            });
        });
    };
    SipPhone.prototype.sipSessionByCallId = function (id) {
        for (var _i = 0, _a = this.allSession; _i < _a.length; _i++) {
            var sess = _a[_i];
            if (sess.callId === id) {
                return sess;
            }
        }
        return null;
    };
    SipPhone.prototype.sipSessionBySipId = function (id) {
        for (var _i = 0, _a = this.allSession; _i < _a.length; _i++) {
            var sess = _a[_i];
            if (sess.id.startsWith(id)) {
                return sess;
            }
        }
        return null;
    };
    SipPhone.prototype.removeSession = function (id, connection) {
        if (connection) {
            var localStreams = connection.getLocalStreams();
            var remoteStreams = connection.getRemoteStreams();
            if (localStreams) {
                localStreams.forEach(function (stream) {
                    stream.getTracks().forEach(function (track) {
                        track.stop();
                    });
                });
            }
            if (remoteStreams) {
                remoteStreams.forEach(function (stream) {
                    stream.getTracks().forEach(function (track) {
                        track.stop();
                    });
                });
            }
        }
        if (this.sessionCache.has(id)) {
            this.sessionCache.delete(id);
            return true;
        }
        return false;
    };
    SipPhone.prototype.storeSession = function (id, session) {
        if (this.sessionCache.has(id)) {
            throw new Error('Session already store');
        }
        this.sessionCache.set(id, session);
    };
    Object.defineProperty(SipPhone.prototype, "allSession", {
        get: function () {
            return Array.from(this.sessionCache.values());
        },
        enumerable: true,
        configurable: true
    });
    SipPhone.prototype.getMediaConstraints = function (req) {
        if (req.screen) {
            return {
                video: false,
                audio: false,
                screen: true,
            };
        }
        return {
            video: req.video || false,
            audio: true,
        };
    };
    SipPhone.prototype.getSession = function (id) {
        if (this.sessionCache.has(id)) {
            return this.sessionCache.get(id);
        }
        return null;
    };
    SipPhone.prototype.hasSession = function (id) {
        return this.sessionCache.has(id);
    };
    SipPhone.prototype.getUserScreen = function () {
        return __awaiter(this, void 0, void 0, function () {
            var _this = this;
            return __generator(this, function (_a) {
                return [2 /*return*/, new Promise(function (resolve, reject) { return __awaiter(_this, void 0, void 0, function () {
                        var stream, e_1;
                        return __generator(this, function (_a) {
                            switch (_a.label) {
                                case 0:
                                    _a.trys.push([0, 2, , 3]);
                                    return [4 /*yield*/, navigator.mediaDevices.getDisplayMedia()];
                                case 1:
                                    stream = (_a.sent());
                                    resolve(stream);
                                    return [3 /*break*/, 3];
                                case 2:
                                    e_1 = _a.sent();
                                    reject(e_1);
                                    return [3 /*break*/, 3];
                                case 3: return [2 /*return*/];
                            }
                        });
                    }); })];
            });
        });
    };
    SipPhone.prototype.getUserCallStream = function (req) {
        return __awaiter(this, void 0, void 0, function () {
            var stream, camera;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        if (!(req.screen === true)) return [3 /*break*/, 2];
                        return [4 /*yield*/, this.getUserScreen()];
                    case 1:
                        stream = _a.sent();
                        _a.label = 2;
                    case 2:
                        if (!(req.audio !== false || req.video)) return [3 /*break*/, 4];
                        return [4 /*yield*/, this.getUserMedia(req.audio, req.video)];
                    case 3:
                        camera = _a.sent();
                        if (stream) {
                            camera.getTracks().forEach(function (track) {
                                stream.addTrack(track);
                            });
                        }
                        else {
                            stream = camera;
                        }
                        _a.label = 4;
                    case 4:
                        if (!stream) {
                            throw new Error('bad stream request');
                        }
                        return [2 /*return*/, stream];
                }
            });
        });
    };
    SipPhone.prototype.getUserMedia = function (audio, video) {
        return __awaiter(this, void 0, void 0, function () {
            var _this = this;
            return __generator(this, function (_a) {
                return [2 /*return*/, new Promise(function (resolve, reject) { return __awaiter(_this, void 0, void 0, function () {
                        var mediaConstraints;
                        return __generator(this, function (_a) {
                            mediaConstraints = {
                                audio: !(audio === false),
                                video: undefined,
                            };
                            if (video) {
                                mediaConstraints.video = {
                                    width: { min: 10, ideal: 1280, max: 1920 },
                                    height: { min: 10, ideal: 720, max: 1080 },
                                    frameRate: { ideal: 10, max: 15 },
                                };
                            }
                            return [2 /*return*/, getMediaStream(mediaConstraints)
                                    .then(resolve)
                                    .catch(reject)];
                        });
                    }); })];
            });
        });
    };
    SipPhone.userAgent = 'Webitel-Phone/0.0.1';
    return SipPhone;
}(EventEmitter));
export { SipPhone };
function getMediaStream(constraints) {
    return __awaiter(this, void 0, void 0, function () {
        return __generator(this, function (_a) {
            return [2 /*return*/, new Promise(function (resolve, reject) {
                    if (navigator.mediaDevices && navigator.mediaDevices.getUserMedia) {
                        navigator.mediaDevices
                            .getUserMedia(constraints)
                            .then(function (stream) { return resolve(stream); })
                            .catch(function (err) { return reject(err); });
                    }
                    else {
                        /// <reference types="webrtc" />
                        var getUserMedia_1 = navigator.getUserMedia ||
                            navigator.webkitGetUserMedia ||
                            navigator.mozGetUserMedia ||
                            navigator.msGetUserMedia;
                        getUserMedia_1(constraints, function (stream) { return resolve(stream); }, function (err) { return reject(err); });
                    }
                })];
        });
    });
}
//# sourceMappingURL=index.js.map